//----------------------------------------------------------------------------------
//	FILE:			DC_md_Adc.c
//	Description:	ADC setup for measurements
//	Version: 		1.0
//  Target:  		TMS320F2806x family
//	Type: 			Device Dependent
//  Copyright       ELK21 course 2011
//  Date:           June 2012
//----------------------------------------------------------------------------------
// TRM = F2806x Technical reference manual

#include "F2806x_Device.h"				// F2806x header file peripheral address definitions

#define CPU_RATE   12.500L   			// for a 80MHz CPU clock speed (SYSCLKOUT)
                             			// 1/80 MHz = 12.5 ns
#define ADC_usDELAY  1000L				// used for delay of 1ms = 1000 us

// DO NOT MODIFY THIS LINE.
//DSP28x_usDelay defined in F2806x_usDelay.asm
#define DELAY_US(A)  DSP28x_usDelay(((((long double) A * 1000.0L) / (long double)CPU_RATE) - 9.0L) / 5.0L)
extern void DSP28x_usDelay(Uint32 Count);


/**********************************************************************
** Description: Initializes the ADC on the F2806x
**********************************************************************/
void InitADC(void)
{
	
	EALLOW;								// Enable EALLOW protected register access
		
// enable ADC clock in case user forgot to enable it in DeviceInit()
SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 1;	// Enable ADC clock

//--- Reset the ADC module
// Note: The ADC is already reset after a DSP reset, but this example is just showing
// good coding practice to reset the peripheral before configuring it as you never
// know why the DSP has started the code over again from the beginning).
//ADC Control Register 1 (ADCCTL1) - refer TRM Table 8-3 page 491
  
	AdcRegs.ADCCTL1.bit.RESET = 1;		// ADC module software reset - Bit 15


// Must wait 2 ADCCLK periods for the reset to take effect.
	asm(" NOP");
	asm(" NOP");
	
//--- Power-up and configure the ADC
// ADC Control Register 1 (ADCCTL1)
// Refer TRM table 8-3 page 491 also power-up sequence page 487 

    AdcRegs.ADCCTL1.bit.ADCPWDN 	= 1;	// ADC power down, 0=powered down, 1=powered up                                       
    AdcRegs.ADCCTL1.bit.ADCBGPWD	= 1;	// ADC bandgap power down, 0=powered down, 1=powered up
    AdcRegs.ADCCTL1.bit.ADCREFPWD	= 1;	// ADC reference power down, 0=powered down, 1=powered up
    AdcRegs.ADCCTL1.bit.ADCREFSEL	= 0;	// ADC reference select, 0=internal, 1=external
    AdcRegs.ADCCTL1.bit.ADCENABLE	= 1;	// 0=disable, 1=Enable ADC - bit 14
                                            
    DELAY_US(ADC_usDELAY);					// Delay - Delay - Must be there for F2806x devices
	
	

    AdcRegs.ADCCTL1.bit.INTPULSEPOS	= 1;	// create int pulses 1 cycle prior to output latch
                                        	// INT pulse generation, 0=start of conversion, 1=end of conversion
  	// For other values, use default ones

//-----ADC Control Register 2 (ADCCTL2
// Refer TRM table 8-4 page 493

	//AdcRegs.ADCCTL2.all = 0x0001;			// ADC clock configuration
	
	AdcRegs.ADCCTL2.bit.CLKDIV4EN=0;      	//ADC clock divider.  0=no effect, 1=CPUCLK/4 if CLKDIV2EN=1 (else no effect)
	AdcRegs.ADCCTL2.bit.ADCNONOVERLAP=0; 	// 0=overlap sample and conversion, 1=no overlap
	AdcRegs.ADCCTL2.bit.CLKDIV2EN=1;     	//ADC clock divider.  0=CPUCLK, 1=CPUCLK/2
                                                       
 
	
//--- SOC0 configuration
// ADC Sample Mode Register - ADCSAMPLEMODE - table 8-11 TRM page 499
  
	AdcRegs.ADCSAMPLEMODE.bit.SIMULEN0 = 0;	// 0=Single sample mode set for SOC0 and SOC1
											// 1=Simultaneous sample for SOC0 and SOC1	                                            
	                                            
    //ADC SOC0 - SOC15 Control Registers (ADCSOCxCTL)
    //TRM table 8-18 page 503
    
    //bit TRIGSEL - SOCx Trigger Source Select.
	AdcRegs.ADCSOC0CTL.bit.TRIGSEL = 5;		// Trigger using ePWM1SOCA
	AdcRegs.ADCSOC1CTL.bit.TRIGSEL = 5;		// Trigger using ePWM1SOCA
	AdcRegs.ADCSOC2CTL.bit.TRIGSEL = 5;		// Trigger using ePWM1SOCA
	AdcRegs.ADCSOC3CTL.bit.TRIGSEL = 5;		// Trigger using ePWM1SOCA
	AdcRegs.ADCSOC4CTL.bit.TRIGSEL = 5;		// Trigger using ePWM1SOCA
	AdcRegs.ADCSOC5CTL.bit.TRIGSEL = 5;		// Trigger using ePWM1SOCA
	AdcRegs.ADCSOC6CTL.bit.TRIGSEL = 5;		// Trigger using ePWM1SOCA
	AdcRegs.ADCSOC7CTL.bit.TRIGSEL = 5;		// Trigger using ePWM1SOCA
	// bit CHSEL - SOCx Channel Select. Selects the channel to be converted when SOCx is received by the ADC.
	AdcRegs.ADCSOC0CTL.bit.CHSEL = 0;		// Convert channel ADCINA0 (ch0)
	AdcRegs.ADCSOC1CTL.bit.CHSEL = 1;		// Convert channel ADCINA1 (ch0)
	AdcRegs.ADCSOC2CTL.bit.CHSEL = 2;		// Convert channel ADCINA2 (ch0)
	AdcRegs.ADCSOC3CTL.bit.CHSEL = 3;		// Convert channel ADCINA3 (ch0)
	AdcRegs.ADCSOC4CTL.bit.CHSEL = 8;		// Convert channel ADCINB0 (ch0)
	AdcRegs.ADCSOC5CTL.bit.CHSEL = 9;		// Convert channel ADCINB1 (ch0)
	AdcRegs.ADCSOC6CTL.bit.CHSEL = 10;		// Convert channel ADCINB2 (ch0)
	AdcRegs.ADCSOC7CTL.bit.CHSEL = 11;		// Convert channel ADCINB3 (ch0)
	// bit ACQPS -SOCx Acquisition Prescale. Controls the sample and hold window for SOCx. 
	 // Minimum value allowed is 6.
	AdcRegs.ADCSOC0CTL.bit.ACQPS = 6;		// Acquisition window set to (6+1)=7 cycles
	AdcRegs.ADCSOC1CTL.bit.ACQPS = 6;		// Acquisition window set to (6+1)=7 cycles
	AdcRegs.ADCSOC2CTL.bit.ACQPS = 6;		// Acquisition window set to (6+1)=7 cycles
	AdcRegs.ADCSOC3CTL.bit.ACQPS = 6;		// Acquisition window set to (6+1)=7 cycles
	AdcRegs.ADCSOC4CTL.bit.ACQPS = 6;		// Acquisition window set to (6+1)=7 cycles
	AdcRegs.ADCSOC5CTL.bit.ACQPS = 6;		// Acquisition window set to (6+1)=7 cycles
	AdcRegs.ADCSOC6CTL.bit.ACQPS = 6;		// Acquisition window set to (6+1)=7 cycles
	AdcRegs.ADCSOC7CTL.bit.ACQPS = 6;		// Acquisition window set to (6+1)=7 cycles
		
	//for calibrating:
	///AdcRegs.ADCOFFTRIM.bit.OFFTRIM=350;

  // ADC Interrupt Trigger SOC Select 1 Register (ADCINTSOCSEL1)
  // TRM table 8-12, page 500
  AdcRegs.ADCINTSOCSEL1.bit.SOC0 = 0;	// No ADCINT triggers SOC0.  TRIGSEL field determines trigger.
//ADC Start of Conversion Priority Control Register (SOCPRICTL)
// TRM table 8-12, page 500
//bit SOCPRIORITY - 
	AdcRegs.SOCPRICTL.bit.SOCPRIORITY = 0;	// All SOCs handled in round-robin mode for all channels

//--- ADCINT1 configuration
// Interrupt Select 1 And 2 Register (INTSEL1N2)
// TRM table 8-9 page 496
 // bit INT1CONT - ADCINTy Continuous Mode Enable
	AdcRegs.INTSEL1N2.bit.INT1CONT = 0;			
	// 1= ADCINTy pulses are generated whenever an EOC pulse is generated irrespective if the flag bit is 
	// cleared or not
	// 0=No further ADCINTx pulses are generated until ADCINTx flag (in ADCINTFLG register) is cleared by user.
   
    // ADCINTy Interrupt Enable                                   
    AdcRegs.INTSEL1N2.bit.INT1E = 1;			// 0=ADCINT1 is disabled, 1=ADCINT1 is enabled.
    //INTySEL - ADCINTy EOC Source Select 
	AdcRegs.INTSEL1N2.bit.INT1SEL = 7;			// EOC7 triggers ADCINT1
	                                            // here you specify which EOC should trigger an interrupt
	                                            
//--- Enable the ADC interrupt - This is done in the main program

	EDIS;						                // Disable EALLOW protected register access

} // end InitAdc()
//--- end of file -----------------------------------------------------
